import {Observable,Subscription,from,switchMap as switchMap$1,concatMap as concatMap$1,mergeMap as mergeMap$1}from'rxjs';/**
 * Copied from https://github.com/felixfbecker/abortable-rx/blob/master/src/index.ts
 * To support esm
 * @returns
 *
 */
const createAbortError = () => {
  const error = new Error('Aborted');
  error.name = 'AbortError';
  return error;
};
/**
 * Creates an Observable just like RxJS `create`, but exposes an AbortSignal in addition to the subscriber
 */
const create = subscribe => new Observable(subscriber => {
  const abortController = new AbortController();
  const subscription = new Subscription();
  const teardown = subscribe && subscribe(subscriber, abortController.signal);
  subscription.add(teardown);
  subscription.add(() => abortController.abort());
  return subscription;
});
/**
 * Easiest way to wrap an abortable async function into a Promise.
 * The factory is called every time the Observable is subscribed to, and the AbortSignal is aborted on unsubscription.
 */
const defer = factory => create((subscriber, signal) => from(factory(signal)).subscribe(subscriber));
/**
 * Returns a Promise that resolves with the last emission of the given Observable,
 * rejects if the Observable errors or rejects with an `AbortError` when the AbortSignal is aborted.
 */
const toPromise = (observable, signal) => new Promise((resolve, reject) => {
  if (signal && signal.aborted) {
    reject(createAbortError());
    return;
  }
  let subscription;
  const listener = () => {
    subscription.unsubscribe();
    reject(createAbortError());
  };
  const cleanup = () => {
    if (signal) {
      signal.removeEventListener('abort', listener);
    }
  };
  let value;
  subscription = observable.subscribe(val => {
    value = val;
  }, err => {
    cleanup();
    reject(err);
  }, () => {
    cleanup();
    resolve(value);
  });
  if (signal) {
    signal.addEventListener('abort', listener, {
      once: true
    });
  }
});
/**
 * Calls `next` for every emission and returns a Promise that resolves when the Observable completed, rejects if the
 * Observable errors or rejects with an `AbortError` when the AbortSignal is aborted.
 */
const forEach = (source, next, signal) => new Promise((resolve, reject) => {
  if (signal && signal.aborted) {
    reject(createAbortError());
    return;
  }
  let subscription;
  const listener = () => {
    subscription.unsubscribe();
    reject(createAbortError());
  };
  const cleanup = () => {
    if (signal) {
      signal.removeEventListener('abort', listener);
    }
  };
  subscription = source.subscribe(value => {
    try {
      next(value);
    } catch (err) {
      reject(err);
      if (subscription) {
        subscription.unsubscribe();
      }
    }
  }, err => {
    cleanup();
    reject(err);
  }, () => {
    cleanup();
    resolve();
  });
  if (signal) {
    signal.addEventListener('abort', listener, {
      once: true
    });
  }
});
/**
 * Like RxJS `switchMap`, but passes an AbortSignal that is aborted when the source emits another element.
 */
const switchMap = project => source => source.pipe(switchMap$1((value, index) => defer(abortSignal => project(value, index, abortSignal))));
/**
 * Like RxJS `concatMap`, but passes an AbortSignal that is aborted when the returned Observable is unsubscribed from.
 */
const concatMap = project => source => source.pipe(concatMap$1((value, index) => defer(abortSignal => project(value, index, abortSignal))));
/**
 * Like RxJS `mergeMap`, but passes an AbortSignal that is aborted when the returned Observable is unsubscribed from.
 */
const mergeMap = project => source => source.pipe(mergeMap$1((value, index) => defer(abortSignal => project(value, index, abortSignal))));export{concatMap,create,defer,forEach,mergeMap,switchMap,toPromise};//# sourceMappingURL=abortable-rx.js.map
